#!/usr/local/bin/perl -w

# Copyright (C) 2006 Ran Friedman
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#                   #
# calc_entropies.pl #
#                   #

#cccccccccccccccccccccccccccccccccccc#
# Ran Friedman, laser laboratory for #
# biological studies, TAU            #
#cccccccccccccccccccccccccccccccccccc#

use strict;
use Getopt::Std;

# Constants
my $k=1.3806503*10**(-23); # joule/K
my $h=1.0545715*10**(-34); # hbar joule*sec
my $hp=6.6260693*10**(-34);
my $u=1.6605402*10**(-27); # Atomic mass unit (kg) 
my $j2c=0.239; # Convert Joule to Cal
my $av=6.022*10**(23); # Avogadro
my $min_ev=1*10**(-6); # smaller eigenvalues are ignored

#
# Declaration and init
#

my $prog="calc_entropies.pl";
our ($opt_h,$opt_i,$opt_t,$opt_n);

#
# Parsing the arguments
#

getopts('hi:t:n');

#
# Business part
#
check_input();

my $infile=$opt_i;
my $t=$opt_t;
my $nma=0;
if ($opt_n) {
  $nma=1;
}

open (IN,$infile)||die "Can't open file $infile: $!\n";

my $s=0; # Entropy
while (<IN>) {
    next if ($_=~/^[@\#]/);
    my @ar=split;
    my $ev=$ar[1];
    my $ev_number=$ar[0];
    next if ($ev<$min_ev);
    next if ($nma && $ev_number<7); # ignore first 6 values for NMA
    if ($nma) {
      my $w=sqrt($ev)*10**12; # ev=(2PI*w)^2, eq. 3.66 GMX 3.2.1
      $s+=-log(1-exp(-$h*$w/($k*$t))) + ($av*$h*$w*exp(-$h*$w/($k*$t))) / ($t*(1-exp(-$h*$w/($k*$t))))
    }
    else {
      my $w=$ev*$u*10**(-18);
      $w=sqrt($k*$t/$w); 
      $s+=$h*$w/($k*$t) / (exp($h*$w/($k*$t))-1)-log(1-exp(-1*$h*$w/($k*$t)));
    }
}
$s=$s*$k*$av;
my $sc=$s*$j2c; # in calK
printf("Sqh\t%.3E (J/mol K)\t %.3E (Cal/mol K)\n",$s,$sc);

close (IN);

#
# subroutines
#

sub check_input {
    # Checks whether the input of the program is correct (i.e
    # all the needed vars are set)
    # If not, prints help message and error message and quits.
    
    if ($opt_h) {
	print_help_message();
    }
    if (!defined($opt_i))  # No input file
    {
	print STDERR "$prog\nUsage:\n\t$prog -i <input - xvg file> -t <temperature> -n (opt.)\n\nFor help, run $prog -h\n";
	exit (1);
    }
    if (!defined($opt_t)) {
	$opt_t=300; # Default temp 300K
    }
    print "$prog runs with the following parameters:\n";
    print "<input - xvg file> - $opt_i\n";
    print "<temperature> - $opt_t\n";
    if ($opt_n) {
      print "Entropies will be calculated for NMA\n";
    }
};

sub print_help_message {
    print "$prog calculates the entropy from the mass-weighted\ncovariance matrix eigenvalues obtained from Gromac\n";
    print "or from NM analysis matrix frequencies (run with -n)\n";
    print "The entropies are calculated based on the quasi-Harmonic approximation\n";
    print "See J Chem Phys 115:6289, Eq. 13\n";
    print "\n$prog was written by:\n\tRan Friedman\n\tLaser Laboratory for Fast Reactions in Biology\n\tDepartment of Biochemistry\n\tThe university of Tel-Aviv\n";
    print "\t",'ran@hemi.tau.ac.il',"\n";
}

0;

